"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.appInsightsLogger = void 0;
var _TelemetryConstants = require("../TelemetryConstants");
var _ScenarioMarker = _interopRequireDefault(require("../ScenarioMarker"));
var _TelemetryHelper = require("../TelemetryHelper");
var _Constants = require("../../Constants");
var _AppInsightsEvents = require("../AppInsightsEvents");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
var AllowedKeys;
(function (AllowedKeys) {
  AllowedKeys["LogLevel"] = "LogLevel";
  AllowedKeys["Description"] = "Description";
  AllowedKeys["ExceptionDetails"] = "ExceptionDetails";
  AllowedKeys["ChannelId"] = "ChannelId";
  AllowedKeys["LCWRuntimeId"] = "ClientSessionId";
  AllowedKeys["ConversationId"] = "LiveWorkItemId";
  AllowedKeys["ChatId"] = "ChatThreadId";
  AllowedKeys["OrganizationId"] = "OrganizationId";
  AllowedKeys["ElapsedTimeInMilliseconds"] = "DurationInMilliseconds";
})(AllowedKeys || (AllowedKeys = {}));
let initializationPromise = null;
const appInsightsLogger = appInsightsKey => {
  const isValidKey = key => {
    const INSTRUMENTATION_KEY_PATTERN = "[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}";
    const INSTRUMENTATION_KEY_REGEX = new RegExp(`^${INSTRUMENTATION_KEY_PATTERN}$`);
    const CONNECTION_STRING_REGEX = new RegExp(`^InstrumentationKey=${INSTRUMENTATION_KEY_PATTERN};IngestionEndpoint=https://[a-zA-Z0-9\\-\\.]+\\.applicationinsights\\.azure\\.com/.*`);
    return INSTRUMENTATION_KEY_REGEX.test(key) || CONNECTION_STRING_REGEX.test(key);
  };
  const initializeAppInsights = async appInsightsKey => {
    if (!window.appInsights && appInsightsKey) {
      if (!isValidKey(appInsightsKey)) {
        _TelemetryHelper.TelemetryHelper.logActionEvent(_TelemetryConstants.LogLevel.ERROR, {
          Event: _TelemetryConstants.TelemetryEvent.AppInsightsInitFailed,
          Description: _Constants.AppInsightsTelemetryMessage.AppInsightsKeyError,
          ExceptionDetails: {
            message: _Constants.AppInsightsTelemetryMessage.AppInsightsKeyError
          }
        });
        return;
      }
      try {
        // Dynamically import Application Insights
        const {
          ApplicationInsights
        } = await Promise.resolve().then(() => _interopRequireWildcard(require("@microsoft/applicationinsights-web")));
        const config = {
          ...(appInsightsKey.includes("IngestionEndpoint") ? {
            connectionString: appInsightsKey
          } : {
            instrumentationKey: appInsightsKey
          })
        };

        // Initialize Application Insights instance
        window.appInsights = new ApplicationInsights({
          config
        });
        window.appInsights.loadAppInsights();
        _TelemetryHelper.TelemetryHelper.logActionEvent(_TelemetryConstants.LogLevel.INFO, {
          Event: _TelemetryConstants.TelemetryEvent.AppInsightsInitialized,
          Description: _Constants.AppInsightsTelemetryMessage.AppInsightsInitialized
        });
      } catch (error) {
        console.error(_Constants.AppInsightsTelemetryMessage.AppInsightsInitError, error);
        _TelemetryHelper.TelemetryHelper.logActionEvent(_TelemetryConstants.LogLevel.ERROR, {
          Event: _TelemetryConstants.TelemetryEvent.AppInsightsInitFailed,
          Description: _Constants.AppInsightsTelemetryMessage.AppInsightsInitError,
          ExceptionDetails: {
            message: `${_Constants.AppInsightsTelemetryMessage.AppInsightsInitError} with key ending: ${appInsightsKey.slice(-3)}`,
            exception: error
          }
        });
      }
    }
  };
  const logger = async () => {
    if (!initializationPromise) {
      initializationPromise = initializeAppInsights(appInsightsKey);
    }
    await initializationPromise;
    return window.appInsights;
  };
  const aiLogger = {
    type: "appInsightsLogger",
    log: async (logLevel, telemetryInput) => {
      try {
        var _telemetryInput$paylo, _telemetryInput$telem;
        const _logger = await logger();
        if (!_logger) return;
        const eventName = telemetryInput === null || telemetryInput === void 0 ? void 0 : (_telemetryInput$paylo = telemetryInput.payload) === null || _telemetryInput$paylo === void 0 ? void 0 : _telemetryInput$paylo.Event;
        const telemetryInfo = telemetryInput === null || telemetryInput === void 0 ? void 0 : (_telemetryInput$telem = telemetryInput.telemetryInfo) === null || _telemetryInput$telem === void 0 ? void 0 : _telemetryInput$telem.telemetryInfo;
        const eventProperties = setEventProperties(telemetryInfo);
        if (eventName) {
          const trackingEventName = getTrackingEventName(logLevel, eventName);
          _logger.trackEvent({
            name: trackingEventName,
            properties: eventProperties
          });
        }
      } catch (error) {
        console.error("Error in logging telemetry to Application Insights:", error);
      }
    },
    dispose: () => {
      if (window.appInsights) {
        window.appInsights.unload(); //flush and unload
        window.appInsights = null;
      }
    }
  };

  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  function setEventProperties(telemetryInfo) {
    const eventProperties = {};
    if (telemetryInfo) {
      for (const key in AllowedKeys) {
        const finalKey = AllowedKeys[key]; // get renamed keys for LCWRuntimeId, ConversationId, ChatId
        const value = telemetryInfo[key];
        if (value !== undefined && value !== null && value !== "") {
          eventProperties[finalKey] = value;
        }
      }
    }
    return eventProperties;
  }
  function getTrackingEventName(logLevel, eventName) {
    // Rename and Remove "UX" or "LCW" prefix if present
    const event = (_AppInsightsEvents.AppInsightsEventMapping[eventName] || eventName).replace(/^(UX|LCW)/, "");
    if (logLevel === _TelemetryConstants.LogLevel.ERROR) {
      return _ScenarioMarker.default.failScenario(event);
    }
    if (logLevel === _TelemetryConstants.LogLevel.WARN) {
      return _ScenarioMarker.default.warnScenario(event);
    }
    if (event.toLowerCase().includes("complete")) {
      return _ScenarioMarker.default.completeScenario(event);
    }
    return _ScenarioMarker.default.startScenario(event);
  }
  return aiLogger;
};
exports.appInsightsLogger = appInsightsLogger;