"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.logWidgetLoadWithUnexpectedError = exports.logWidgetLoadComplete = exports.handleStartChatError = void 0;
var _omnichannelChatSdk = require("@microsoft/omnichannel-chat-sdk");
var _TelemetryConstants = require("../../../common/telemetry/TelemetryConstants");
var _Constants = require("../../../common/Constants");
var _endChat = require("./endChat");
var _ConversationState = require("../../../contexts/common/ConversationState");
var _DataStoreManager = require("../../../common/contextDataStore/DataStoreManager");
var _LiveChatWidgetActionType = require("../../../contexts/common/LiveChatWidgetActionType");
var _StartChatFailureType = require("../../../contexts/common/StartChatFailureType");
var _TelemetryHelper = require("../../../common/telemetry/TelemetryHelper");
var _TelemetryManager = require("../../../common/telemetry/TelemetryManager");
var _utils = require("../../../common/utils");
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const handleStartChatError = (dispatch, facadeChatSDK, props, ex, isStartChatSuccessful) => {
  var _props$controlProps;
  if (!ex) {
    logWidgetLoadFailed();
    return;
  }

  // Handle internal or misc errors
  if (ex.message === _Constants.WidgetLoadCustomErrorString.AuthenticationFailedErrorString) {
    dispatch({
      type: _LiveChatWidgetActionType.LiveChatWidgetActionType.SET_START_CHAT_FAILURE_TYPE,
      payload: _StartChatFailureType.StartChatFailureType.AuthSetupError
    });
    // set conversation to error to enforce error UI pane
    dispatch({
      type: _LiveChatWidgetActionType.LiveChatWidgetActionType.SET_CONVERSATION_STATE,
      payload: _ConversationState.ConversationState.Error
    });
    logWidgetLoadCompleteWithError(ex);
  }
  if (ex.message === _Constants.WidgetLoadCustomErrorString.NetworkErrorString) {
    logWidgetLoadCompleteWithError(ex);
  }

  // Handle ChatSDK errors
  if (ex instanceof _omnichannelChatSdk.ChatSDKError) {
    switch (ex.message) {
      case _omnichannelChatSdk.ChatSDKErrorName.WidgetUseOutsideOperatingHour:
        handleWidgetUseOutsideOperatingHour(dispatch);
        return;
      case _omnichannelChatSdk.ChatSDKErrorName.PersistentChatConversationRetrievalFailure:
        handlePersistentChatConversationRetrievalFailure(ex);
        break;
      case _omnichannelChatSdk.ChatSDKErrorName.ConversationInitializationFailure:
        handleConversationInitializationFailure(ex);
        break;
      case _omnichannelChatSdk.ChatSDKErrorName.ChatTokenRetrievalFailure:
        handleChatTokenRetrievalFailure(dispatch, ex);
        break;
      case _omnichannelChatSdk.ChatSDKErrorName.UninitializedChatSDK:
        handleUninitializedChatSDK(ex);
        break;
      // Handle the case indicating failure to retrieve an authenticated chat conversation 
      case _omnichannelChatSdk.ChatSDKErrorName.AuthenticatedChatConversationRetrievalFailure:
        logWidgetLoadCompleteWithError(ex);
        break;
      case _omnichannelChatSdk.ChatSDKErrorName.InvalidConversation:
      case _omnichannelChatSdk.ChatSDKErrorName.ClosedConversation:
        handleInvalidOrClosedConversation(dispatch, facadeChatSDK, props, ex);
        return;
      default:
        logWidgetLoadFailed(ex);
    }
  }

  // Show the error UI pane
  dispatch({
    type: _LiveChatWidgetActionType.LiveChatWidgetActionType.SET_START_CHAT_FAILING,
    payload: true
  });
  if (!(props !== null && props !== void 0 && (_props$controlProps = props.controlProps) !== null && _props$controlProps !== void 0 && _props$controlProps.hideErrorUIPane)) {
    // New flow of leveraging ConversationState.Error
    // Set app state to failing start chat if hideErrorUI is not turned on
    _TelemetryHelper.TelemetryHelper.logLoadingEvent(_TelemetryConstants.LogLevel.INFO, {
      Event: _TelemetryConstants.TelemetryEvent.ErrorUIPaneLoaded,
      Description: "Error UI Pane Loaded"
    });
    dispatch({
      type: _LiveChatWidgetActionType.LiveChatWidgetActionType.SET_CONVERSATION_STATE,
      payload: _ConversationState.ConversationState.Error
    });
  } else {
    // Old flow of leveraging ConversationState.Loading
    // Show the loading pane in other cases for failure, this will help for both hideStartChatButton case
    dispatch({
      type: _LiveChatWidgetActionType.LiveChatWidgetActionType.SET_CONVERSATION_STATE,
      payload: _ConversationState.ConversationState.Loading
    });
  }

  // If sessionInit was successful but LCW startchat failed due to some reason e.g adapter didn't load
  // we need to directly endChat to avoid leaving ghost chats in OC, not disturbing any other UI state
  if (isStartChatSuccessful === true) {
    forceEndChat(facadeChatSDK);
  }
};
exports.handleStartChatError = handleStartChatError;
const logWidgetLoadFailed = ex => {
  var _TelemetryTimers$Widg;
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  const exDetails = {
    Exception: `Widget load complete with error: ${ex}`
  };
  if (ex !== null && ex !== void 0 && ex.httpResponseStatusCode) {
    exDetails.HttpResponseStatusCode = ex.httpResponseStatusCode;
  }
  _TelemetryHelper.TelemetryHelper.logLoadingEventToAllTelemetry(_TelemetryConstants.LogLevel.ERROR, {
    Event: _TelemetryConstants.TelemetryEvent.WidgetLoadFailed,
    Description: "Widget load complete with error",
    ExceptionDetails: exDetails,
    ElapsedTimeInMilliseconds: _TelemetryManager.TelemetryTimers === null || _TelemetryManager.TelemetryTimers === void 0 ? void 0 : (_TelemetryTimers$Widg = _TelemetryManager.TelemetryTimers.WidgetLoadTimer) === null || _TelemetryTimers$Widg === void 0 ? void 0 : _TelemetryTimers$Widg.milliSecondsElapsed
  });
};
const logWidgetLoadComplete = additionalMessage => {
  var _TelemetryTimers$Widg2;
  let descriptionString = "Widget load complete";
  if (additionalMessage) {
    descriptionString += `. ${additionalMessage}`;
  }
  _TelemetryHelper.TelemetryHelper.logLoadingEventToAllTelemetry(_TelemetryConstants.LogLevel.INFO, {
    Event: _TelemetryConstants.TelemetryEvent.WidgetLoadComplete,
    Description: descriptionString,
    ElapsedTimeInMilliseconds: _TelemetryManager.TelemetryTimers === null || _TelemetryManager.TelemetryTimers === void 0 ? void 0 : (_TelemetryTimers$Widg2 = _TelemetryManager.TelemetryTimers.WidgetLoadTimer) === null || _TelemetryTimers$Widg2 === void 0 ? void 0 : _TelemetryTimers$Widg2.milliSecondsElapsed
  });
};
exports.logWidgetLoadComplete = logWidgetLoadComplete;
const logWidgetLoadCompleteWithError = ex => {
  var _TelemetryTimers$Widg3;
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  const exDetails = {
    Exception: `Widget load complete with error: ${ex}`
  };
  if (ex !== null && ex !== void 0 && ex.httpResponseStatusCode) {
    exDetails.HttpResponseStatusCode = ex.httpResponseStatusCode;
  }
  _TelemetryHelper.TelemetryHelper.logLoadingEventToAllTelemetry(_TelemetryConstants.LogLevel.ERROR, {
    Event: _TelemetryConstants.TelemetryEvent.WidgetLoadFailed,
    Description: "Widget load complete with error",
    ExceptionDetails: exDetails,
    ElapsedTimeInMilliseconds: _TelemetryManager.TelemetryTimers === null || _TelemetryManager.TelemetryTimers === void 0 ? void 0 : (_TelemetryTimers$Widg3 = _TelemetryManager.TelemetryTimers.WidgetLoadTimer) === null || _TelemetryTimers$Widg3 === void 0 ? void 0 : _TelemetryTimers$Widg3.milliSecondsElapsed
  });
};
const logWidgetLoadWithUnexpectedError = ex => {
  var _TelemetryTimers$Widg4;
  // eslint-disable-line @typescript-eslint/no-explicit-any
  const details = {
    message: (ex === null || ex === void 0 ? void 0 : ex.message) || "An unexpected error occurred",
    stack: (ex === null || ex === void 0 ? void 0 : ex.stack) || "No stack trace available"
  };
  let additionalDetails = "";
  try {
    additionalDetails = JSON.stringify(details);
  } catch (error) {
    additionalDetails = "Failed to stringify error details";
  }

  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  const exDetails = {
    Exception: `Widget load with unexpected error: ${additionalDetails}`
  };
  if (ex !== null && ex !== void 0 && ex.httpResponseStatusCode) {
    exDetails.HttpResponseStatusCode = ex.httpResponseStatusCode;
  }
  _TelemetryHelper.TelemetryHelper.logLoadingEventToAllTelemetry(_TelemetryConstants.LogLevel.ERROR, {
    Event: _TelemetryConstants.TelemetryEvent.WidgetLoadFailed,
    Description: "Widget load with unexpected error",
    ExceptionDetails: exDetails,
    ElapsedTimeInMilliseconds: _TelemetryManager.TelemetryTimers === null || _TelemetryManager.TelemetryTimers === void 0 ? void 0 : (_TelemetryTimers$Widg4 = _TelemetryManager.TelemetryTimers.WidgetLoadTimer) === null || _TelemetryTimers$Widg4 === void 0 ? void 0 : _TelemetryTimers$Widg4.milliSecondsElapsed
  });
};

// eslint-disable-next-line @typescript-eslint/no-explicit-any
exports.logWidgetLoadWithUnexpectedError = logWidgetLoadWithUnexpectedError;
const forceEndChat = facadeChatSDK => {
  _TelemetryHelper.TelemetryHelper.logSDKEvent(_TelemetryConstants.LogLevel.INFO, {
    Event: _TelemetryConstants.TelemetryEvent.PrepareEndChat,
    Description: _Constants.PrepareEndChatDescriptionConstants.WidgetLoadFailedAfterSessionInit
  });
  _TelemetryHelper.TelemetryHelper.logSDKEvent(_TelemetryConstants.LogLevel.INFO, {
    Event: _TelemetryConstants.TelemetryEvent.EndChatSDKCall
  });
  facadeChatSDK === null || facadeChatSDK === void 0 ? void 0 : facadeChatSDK.getChatSDK().endChat();
};
const handleWidgetUseOutsideOperatingHour = dispatch => {
  dispatch({
    type: _LiveChatWidgetActionType.LiveChatWidgetActionType.SET_OUTSIDE_OPERATING_HOURS,
    payload: true
  });
  dispatch({
    type: _LiveChatWidgetActionType.LiveChatWidgetActionType.SET_CONVERSATION_STATE,
    payload: _ConversationState.ConversationState.OutOfOffice
  });
  logWidgetLoadComplete(_Constants.WidgetLoadTelemetryMessage.OOOHMessage);
};

// eslint-disable-next-line @typescript-eslint/no-explicit-any
const handlePersistentChatConversationRetrievalFailure = ex => {
  if (ex.httpResponseStatusCode === 400) {
    logWidgetLoadFailed(ex);
  } else {
    logWidgetLoadCompleteWithError(ex);
  }
};

// eslint-disable-next-line @typescript-eslint/no-explicit-any
const handleConversationInitializationFailure = ex => {
  if (ex.httpResponseStatusCode === 400) {
    logWidgetLoadFailed(ex);
  } else {
    logWidgetLoadCompleteWithError(ex);
  }
};

// eslint-disable-next-line @typescript-eslint/no-explicit-any
const handleChatTokenRetrievalFailure = (dispatch, ex) => {
  if (ex.httpResponseStatusCode === 400) {
    logWidgetLoadFailed(ex);
  } else {
    if (ex.httpResponseStatusCode === 401) {
      dispatch({
        type: _LiveChatWidgetActionType.LiveChatWidgetActionType.SET_START_CHAT_FAILURE_TYPE,
        payload: _StartChatFailureType.StartChatFailureType.Unauthorized
      });
    }
    logWidgetLoadCompleteWithError(ex);
  }
};

// eslint-disable-next-line @typescript-eslint/no-explicit-any
const handleUninitializedChatSDK = ex => {
  logWidgetLoadCompleteWithError(ex);
};

// eslint-disable-next-line @typescript-eslint/no-explicit-any
const handleInvalidOrClosedConversation = (dispatch, facadeChatSDK, props, ex) => {
  var _DataStoreManager$cli;
  logWidgetLoadCompleteWithError(ex);

  // Reset all internal states
  (0, _endChat.callingStateCleanUp)(dispatch);
  (0, _endChat.endChatStateCleanUp)(dispatch);
  (0, _endChat.closeChatStateCleanUp)(dispatch);
  (0, _endChat.chatSDKStateCleanUp)(facadeChatSDK.getChatSDK());
  (_DataStoreManager$cli = _DataStoreManager.DataStoreManager.clientDataStore) === null || _DataStoreManager$cli === void 0 ? void 0 : _DataStoreManager$cli.removeData((0, _utils.getWidgetCacheIdfromProps)(props));

  // Starts new chat
  dispatch({
    type: _LiveChatWidgetActionType.LiveChatWidgetActionType.SET_CONVERSATION_STATE,
    payload: _ConversationState.ConversationState.Closed
  });
};