"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createTrackingForFirstMessage = void 0;
var _TelemetryConstants = require("../common/telemetry/TelemetryConstants");
var _omnichannelChatComponents = require("@microsoft/omnichannel-chat-components");
var _TelemetryHelper = require("../common/telemetry/TelemetryHelper");
var _util = require("./util");
// This tracker is event based, this is since we are tracking events coming from different sources
//  with different timeline, therefore this is a functional approach to track the events, instead of a class based approach
const createTrackingForFirstMessage = () => {
  // Reset the tracking variables
  let startTracking = false;
  let stopTracking = false;
  let startTime = 0;
  let stopTime = 0;
  let stopTrackingMessage;
  let flag = false;
  const isMessageFromValidSender = payload => {
    var _payload$tags;
    // agent scenario
    if (payload !== null && payload !== void 0 && (_payload$tags = payload.tags) !== null && _payload$tags !== void 0 && _payload$tags.includes("public")) {
      return false;
    }
    return true;
  };
  const widgetLoadListener = _omnichannelChatComponents.BroadcastService.getMessageByEventName(_TelemetryConstants.TelemetryEvent.WidgetLoadComplete).subscribe(() => {
    if (startTracking) return;
    startTracking = true;
    startTime = new Date().getTime();
  });
  const newMessageListener = _omnichannelChatComponents.BroadcastService.getMessageByEventName(_TelemetryConstants.BroadcastEvent.NewMessageReceived).subscribe(message => {
    const payload = message.payload;

    // we only care for bot, so we need to check if the message is from the bot
    // pending to add typing message indicator signal detection

    if (isMessageFromValidSender(payload)) {
      if (startTracking && !stopTracking) {
        stopTime = new Date().getTime();
        const elapsedTime = stopTime - startTime;
        stopTracking = true;
        stopTrackingMessage = (0, _util.createTrackingMessage)(payload, "botMessage");
        notifyFMLTrackingCompleted();
        _TelemetryHelper.TelemetryHelper.logActionEvent(_TelemetryConstants.LogLevel.INFO, {
          Event: _TelemetryConstants.TelemetryEvent.BotFirstMessageLapTrack,
          Description: "First Message from Bot latency tracking",
          CustomProperties: {
            elapsedTime,
            widgetLoadedAt: startTime,
            botMessage: stopTrackingMessage
          }
        });
      }
    }

    // this track only first message, if coming from the bot or not
    // the only difference is that it logs only those from bot
    disconnectListener();
  });
  const notifyFMLTrackingCompleted = () => {
    ackListener();
    // Retry sending until flag is true, but do not block the main thread
    const interval = setInterval(() => {
      if (flag) {
        clearInterval(interval);
      } else {
        _omnichannelChatComponents.BroadcastService.postMessage({
          eventName: _TelemetryConstants.BroadcastEvent.FMLTrackingCompleted,
          payload: null
        });
      }
    }, 100);
  };
  const ackListener = () => {
    const listen = _omnichannelChatComponents.BroadcastService.getMessageByEventName(_TelemetryConstants.BroadcastEvent.FMLTrackingCompletedAck).subscribe(() => {
      flag = true;
      listen.unsubscribe();
    });
  };

  // Rehydrate message is received when the widget is reloaded, this is to ensure that we are not tracking messages that are not part of the current conversation
  // No need to keep listerning for tracking, enforcing disconnection for the listners
  const rehydrateListener = _omnichannelChatComponents.BroadcastService.getMessageByEventName(_TelemetryConstants.TelemetryEvent.RehydrateMessageReceived).subscribe(() => {
    startTracking = false;
    stopTracking = false;
    disconnectListener();
  });

  // Rehydrate message is received when the widget is reloaded, this is to ensure that we are not tracking messages that are not part of the current conversation
  // No need to keep listerning for tracking, enforcing disconnection for the listners
  const historyListener = _omnichannelChatComponents.BroadcastService.getMessageByEventName(_TelemetryConstants.BroadcastEvent.HistoryMessageReceived).subscribe(() => {
    startTracking = false;
    stopTracking = false;
    disconnectListener();
  });
  const offlineNetworkListener = _omnichannelChatComponents.BroadcastService.getMessageByEventName(_TelemetryConstants.TelemetryEvent.NetworkDisconnected).subscribe(() => {
    startTracking = false;
    stopTracking = false;
    disconnectListener();
    _TelemetryHelper.TelemetryHelper.logActionEvent(_TelemetryConstants.LogLevel.INFO, {
      Event: _TelemetryConstants.TelemetryEvent.BotFirstMessageLapTrackError,
      Description: "Tracker Stopped due to network disconnection"
    });
  });

  // this is to ensure that we are not tracking messages that are not part of the current conversation
  const disconnectListener = () => {
    historyListener.unsubscribe();
    rehydrateListener.unsubscribe();
    newMessageListener.unsubscribe();
    widgetLoadListener.unsubscribe();
    offlineNetworkListener.unsubscribe();
  };
};
exports.createTrackingForFirstMessage = createTrackingForFirstMessage;