/* eslint-disable @typescript-eslint/no-explicit-any */
/******
 * AttachmentMiddleware
 *
 * Handles attachment downloading.
 ******/

import { Constants, MimeTypes, WebChatMiddlewareConstants } from "../../../../../common/Constants";
import React from "react";
import { getFileAttachmentIconData, isInlineMediaSupported } from "../../../common/utils/FileAttachmentIconManager";
import Attachment from "./attachments/Attachment";
import { BroadcastEvent } from "../../../../../common/telemetry/TelemetryConstants";
import { BroadcastService } from "@microsoft/omnichannel-chat-components";
import FileScanStatus from "./attachments/FileScanStatus";
import MaliciousAttachment from "./attachments/MaliciousAttachment";
import { NotificationHandler } from "../../notification/NotificationHandler";
import { NotificationScenarios } from "../../enums/NotificationScenarios";
import ScanInProgressAttachment from "./attachments/ScanInProgressAttachment";
import { WebChatActionType } from "../../enums/WebChatActionType";
import { defaultAttachmentAdaptiveCardStyles } from "./defaultStyles/defaultAtttachmentAdaptiveCardStyles";
import { defaultAttachmentProps } from "../../../common/defaultProps/defaultAttachmentProps";
import { defaultMiddlewareLocalizedTexts } from "../../../common/defaultProps/defaultMiddlewareLocalizedTexts";
import { useChatContextStore } from "../../../../..";

/**
* Patch card with different attachment data.
* @param card
* @param newAttachment
*/
const patchAttachment = (card, newAttachment) => {
  const {
    activity,
    attachment
  } = card;
  const patchedAttachment = Object.assign({}, attachment);
  patchedAttachment.contentType = newAttachment.contentType;
  patchedAttachment.thumbnailUrl = newAttachment.thumbnailUrl;
  const patchedAttachments = activity.attachments.map(target => target === attachment ? patchedAttachment : target);
  const patchedActivity = Object.assign({}, activity);
  patchedActivity.attachments = patchedAttachments;
  return {
    activity: patchedActivity,
    attachment: patchedAttachment
  };
};
const genPreviewCardWithAttachment = (card, iconData, next) => {
  const patchedTextCard = patchAttachment(card, {
    contentType: MimeTypes.UnknownFileType,
    thumbnailUrl: undefined
  });
  return /*#__PURE__*/React.createElement(Attachment, {
    iconData: iconData,
    imageCard: card,
    textCard: patchedTextCard,
    renderer: next
  });
};
export const createAttachmentMiddleware = enableInlinePlaying => {
  // eslint-disable-next-line react/display-name
  const attachmentMiddleware = () => next => function () {
    var _state$domainStates$r, _state$domainStates$r2, _state$domainStates$r3, _card$activity$channe;
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }
    const [card] = args;
    if (!(card !== null && card !== void 0 && card.activity)) {
      return next(...args);
    }
    const {
      activity: {
        attachments
      },
      attachment
    } = card;
    // No attachment
    if (!attachments || !attachments.length || !attachment) {
      return next(...args);
    }

    // Check for Adaptive cards
    // eslint-disable-next-line prefer-const
    let {
      content,
      contentType
    } = attachment || {
      content: "",
      contentType: ""
    };
    let {
      type
    } = content || {
      type: ""
    };
    if (!type && content && Constants.supportedAdaptiveCardContentTypes.indexOf(contentType) >= 0) {
      try {
        content = JSON.parse(content);
        type = content.type;
        card.attachment.content = content;
      } catch (e) {
        const errorData = "Unable to parse the adaptive card format";
        BroadcastService.postMessage({
          eventName: BroadcastEvent.InvalidAdaptiveCardFormat,
          payload: {
            Message: errorData,
            ExceptionDetails: e
          }
        });
      }
    }
    const [state] = useChatContextStore();
    const attachmentId = ((_state$domainStates$r = state.domainStates.renderingMiddlewareProps) === null || _state$domainStates$r === void 0 ? void 0 : (_state$domainStates$r2 = _state$domainStates$r.attachmentProps) === null || _state$domainStates$r2 === void 0 ? void 0 : _state$domainStates$r2.adaptiveCardAttachmentId) ?? defaultAttachmentProps.adaptiveCardAttachmentId;
    const atttachmentAdaptiveCardStyles = {
      ...defaultAttachmentAdaptiveCardStyles,
      ...((_state$domainStates$r3 = state.domainStates.renderingMiddlewareProps) === null || _state$domainStates$r3 === void 0 ? void 0 : _state$domainStates$r3.attachmentAdaptiveCardStyles)
    };
    if (type === WebChatMiddlewareConstants.adaptiveCard || Constants.supportedAdaptiveCardContentTypes.indexOf(contentType) >= 0) {
      return /*#__PURE__*/React.createElement("div", {
        id: attachmentId,
        style: atttachmentAdaptiveCardStyles
      }, next(...args));
    } else if (contentType.startsWith(Constants.adaptiveCardContentTypePrefix)) {
      console.warn(`${contentType} adaptive card type is currently not supported.`);
    }
    if ((_card$activity$channe = card.activity.channelData) !== null && _card$activity$channe !== void 0 && _card$activity$channe.middlewareData) {
      attachment.contentUrl = card.activity.channelData.middlewareData[attachment.name];
    } else if (attachment !== null && attachment !== void 0 && attachment.tempContentUrl) {
      attachment.contentUrl = attachment.tempContentUrl;
    }
    if (!attachment.name) {
      return next(...args);
    }
    if (card.activity.channelData && card.activity.channelData.fileScan) {
      const index = attachments.findIndex(attachment => attachment.name === card.attachment.name);
      const {
        activity: {
          channelData: {
            fileScan
          }
        }
      } = card;
      const scanResult = fileScan[index];
      if ((scanResult === null || scanResult === void 0 ? void 0 : scanResult.status) === FileScanStatus.INPROGRESS) {
        return /*#__PURE__*/React.createElement(ScanInProgressAttachment, {
          textCard: card
        });
      }
      if ((scanResult === null || scanResult === void 0 ? void 0 : scanResult.status) === FileScanStatus.MALWARE) {
        var _state$domainStates$m;
        const localizedText = ((_state$domainStates$m = state.domainStates.middlewareLocalizedTexts) === null || _state$domainStates$m === void 0 ? void 0 : _state$domainStates$m.MIDDLEWARE_BANNER_FILE_IS_MALICIOUS) ?? defaultMiddlewareLocalizedTexts.MIDDLEWARE_BANNER_FILE_IS_MALICIOUS;
        NotificationHandler.notifyError(NotificationScenarios.AttachmentError, localizedText.replace("{0}", attachment.name));
        return /*#__PURE__*/React.createElement(MaliciousAttachment, {
          textCard: card
        });
      }
    }
    const fileExtension = attachment.name.substring(attachment.name.lastIndexOf(".") + 1, attachment.name.length) || attachment.name;
    const imageExtension = Constants.imageRegex.test(attachment.name);
    const audioExtension = Constants.audioMediaRegex.test(attachment.name);
    const videoExtension = Constants.videoMediaRegex.test(attachment.name);
    const iconData = getFileAttachmentIconData(fileExtension);
    if (imageExtension) {
      return genPreviewCardWithAttachment(card, iconData, next);
    }
    if (audioExtension || videoExtension) {
      if (enableInlinePlaying && card.activity.actionType && card.activity.actionType === WebChatActionType.DIRECT_LINE_INCOMING_ACTIVITY && isInlineMediaSupported(attachment.name)) {
        return genPreviewCardWithAttachment(card, iconData, next);
      }
      return /*#__PURE__*/React.createElement(Attachment, {
        iconData: iconData,
        textCard: patchAttachment(card, {
          contentType: MimeTypes.UnknownFileType,
          thumbnailUrl: undefined
        }),
        renderer: next
      });
    }
    const isUnknownImageObject = contentType.toLowerCase().includes("image") && !imageExtension;
    if (fileExtension === "txt" || isUnknownImageObject) {
      return /*#__PURE__*/React.createElement(Attachment, {
        iconData: iconData,
        textCard: patchAttachment(card, {
          contentType: MimeTypes.UnknownFileType,
          thumbnailUrl: undefined
        }),
        renderer: next
      });
    }
    return /*#__PURE__*/React.createElement(Attachment, {
      iconData: iconData,
      textCard: card,
      renderer: next
    });
  };
  return attachmentMiddleware;
};