/******
 * HTMLTextMiddleware
 * 
 * Ensures that links within messages are processed so that the caller website cannot be traced.
 ******/

import { Constants, HtmlAttributeNames } from "../../../../../common/Constants";
import DOMPurify from "dompurify";
import { WebChatActionType } from "../../enums/WebChatActionType";
import { TelemetryHelper } from "../../../../../common/telemetry/TelemetryHelper";
import { LogLevel, TelemetryEvent } from "../../../../../common/telemetry/TelemetryConstants";
import updateIn from "simple-update-in";

// eslint-disable-next-line @typescript-eslint/no-explicit-any
const convertTextToHtmlNode = text => {
  if (!text) return "";
  const element = document.createElement(HtmlAttributeNames.div);
  try {
    text = DOMPurify.sanitize(text, {
      ADD_ATTR: ["target"]
    });
    element.innerHTML = text;
  } catch (e) {
    const errorMessage = `Failed to purify and set innertHTML with text: ${text}`;
    TelemetryHelper.logActionEvent(LogLevel.ERROR, {
      Event: TelemetryEvent.ProcessingHTMLTextMiddlewareFailed,
      ExceptionDetails: {
        ErrorData: errorMessage,
        Exception: e
      }
    });
  }
  return element;
};

// eslint-disable-next-line @typescript-eslint/no-explicit-any
const processHTMLText = (action, text, honorsTargetInHTMLLinks) => {
  var _htmlNode$tagName;
  const htmlNode = convertTextToHtmlNode(text);
  const aNodes = htmlNode.getElementsByTagName(HtmlAttributeNames.aTagName);
  if ((aNodes === null || aNodes === void 0 ? void 0 : aNodes.length) > 0) {
    try {
      for (let index = 0; index < aNodes.length; index++) {
        var _aNode$tagName;
        const aNode = aNodes[index];
        // Skip if the node is not valid or the node's tag name is not equal to 'a', or the node href is empty.
        if (((_aNode$tagName = aNode.tagName) === null || _aNode$tagName === void 0 ? void 0 : _aNode$tagName.toLowerCase()) !== HtmlAttributeNames.aTagName || !aNode.href) {
          continue;
        }

        // Add target to 'a' node if target is missing or if honorsTargetInHTMLLinks is false
        if (!aNode.target || !honorsTargetInHTMLLinks) {
          aNode.target = Constants.blank;
        }

        // If rel is missing or rel does not include noopener and noreferrer, add them
        if (!aNode.rel) {
          aNode.rel = `${HtmlAttributeNames.noopenerTag} ${HtmlAttributeNames.noreferrerTag}`;
        } else {
          if (aNode.rel.indexOf(HtmlAttributeNames.noopenerTag) === -1) {
            aNode.rel += ` ${HtmlAttributeNames.noopenerTag}`;
          }
          if (aNode.rel.indexOf(HtmlAttributeNames.noreferrerTag) === -1) {
            aNode.rel += ` ${HtmlAttributeNames.noreferrerTag}`;
          }
        }
      }
      action = updateIn(action, [Constants.payload, Constants.activity, Constants.text], () => htmlNode.innerHTML);
    } catch (e) {
      let errorMessage = "Failed to apply action: ";
      try {
        errorMessage += JSON.stringify(action);
      } catch (e) {
        errorMessage += " (unable to stringify action)";
      }
      TelemetryHelper.logActionEvent(LogLevel.ERROR, {
        Event: TelemetryEvent.ProcessingHTMLTextMiddlewareFailed,
        ExceptionDetails: {
          ErrorData: errorMessage,
          Exception: e
        }
      });
    }
  }

  // if empty div tag after sanitization
  if (((_htmlNode$tagName = htmlNode.tagName) === null || _htmlNode$tagName === void 0 ? void 0 : _htmlNode$tagName.toLowerCase()) === HtmlAttributeNames.div && htmlNode.children.length === 0 && htmlNode.innerHTML.trim() === "") {
    action = updateIn(action, [Constants.payload, Constants.activity, Constants.text], () => "");
  }
  return action;
};

// eslint-disable-next-line @typescript-eslint/no-explicit-any, @typescript-eslint/no-unused-vars
const htmlTextMiddleware = honorsTargetInHTMLLinks => _ref => {
  let {
    dispatch
  } = _ref;
  return next => action => {
    if (action.type === WebChatActionType.DIRECT_LINE_INCOMING_ACTIVITY) {
      try {
        var _action$payload, _action$payload$activ;
        const text = (_action$payload = action.payload) === null || _action$payload === void 0 ? void 0 : (_action$payload$activ = _action$payload.activity) === null || _action$payload$activ === void 0 ? void 0 : _action$payload$activ.text;
        if (text) {
          action = processHTMLText(action, text, honorsTargetInHTMLLinks ?? false);
        }
      } catch (e) {
        let errorMessage = "Failed to validate action.";
        try {
          errorMessage += JSON.stringify(action);
        } catch (e) {
          errorMessage += " (unable to stringify action)";
        }
        TelemetryHelper.logActionEvent(LogLevel.ERROR, {
          Event: TelemetryEvent.ProcessingHTMLTextMiddlewareFailed,
          ExceptionDetails: {
            ErrorData: errorMessage,
            Exception: e
          }
        });
      }
    }
    return next(action);
  };
};
export default htmlTextMiddleware;