import { BroadcastEvent, LogLevel, TelemetryEvent } from "../common/telemetry/TelemetryConstants";
import { ScenarioType } from "../firstresponselatency/Constants";
import { buildMessagePayload, getScenarioType, isHistoryMessage, polyfillMessagePayloadForEvent } from "../firstresponselatency/util";
import { BroadcastService } from "@microsoft/omnichannel-chat-components";
import { Constants } from "../common/Constants";
import { FirstResponseLatencyTracker } from "../firstresponselatency/FirstResponseLatencyTracker";
import { TelemetryHelper } from "../common/telemetry/TelemetryHelper";
import { TelemetryManager } from "../common/telemetry/TelemetryManager";
export const createOnNewAdapterActivityHandler = (chatId, userId) => {
  // Hooking the message tracker in the listener, a bit invasive but easier to control.
  const firstResponseLatencyTracker = new FirstResponseLatencyTracker();
  // epoch time in utc for when start to listen.
  // We dont longer have a mechanism to know if a message is history or new, so any message older than the time we start listening will be considered a history message.
  // this is a workaround for the fact that we dont have a way to identify if a message is history or new, and it will provide consistency across different scenarios
  const startTime = new Date().getTime();
  let isHistoryMessageReceivedEventRaised = false;
  const onNewAdapterActivityHandler = activity => {
    raiseMessageEvent(activity);
  };
  const userSendMessageStrategy = activity => {
    var _TelemetryManager$Int;
    const payload = buildMessagePayload(activity, userId);
    payload.messageType = Constants.userMessageTag;
    const newMessageSentEvent = {
      eventName: BroadcastEvent.NewMessageSent,
      payload: polyfillMessagePayloadForEvent(activity, payload, (_TelemetryManager$Int = TelemetryManager.InternalTelemetryData) === null || _TelemetryManager$Int === void 0 ? void 0 : _TelemetryManager$Int.conversationId)
    };
    BroadcastService.postMessage(newMessageSentEvent);
    if (!isHistoryMessage(activity, startTime)) {
      firstResponseLatencyTracker.startClock(payload);
    }
    TelemetryHelper.logActionEventToAllTelemetry(LogLevel.INFO, {
      Event: TelemetryEvent.MessageSent,
      Description: "New message sent"
    });
  };
  const systemMessageStrategy = activity => {
    const payload = buildMessagePayload(activity, userId);
    payload.messageType = Constants.systemMessageTag;
    if (isHistoryMessage(activity, startTime)) {
      var _TelemetryManager$Int2;
      historyMessageStrategy(polyfillMessagePayloadForEvent(activity, payload, (_TelemetryManager$Int2 = TelemetryManager.InternalTelemetryData) === null || _TelemetryManager$Int2 === void 0 ? void 0 : _TelemetryManager$Int2.conversationId));
      return;
    }
    TelemetryHelper.logActionEventToAllTelemetry(LogLevel.INFO, {
      Event: TelemetryEvent.SystemMessageReceived,
      Description: "System message received"
    });
  };
  const historyMessageStrategy = payload => {
    const newMessageReceivedEvent = {
      eventName: BroadcastEvent.HistoryMessageReceived,
      payload: payload
    };
    BroadcastService.postMessage(newMessageReceivedEvent);
    if (!isHistoryMessageReceivedEventRaised) {
      // this is needed for reload scenarios, it helps to identify the last message received before the reload
      isHistoryMessageReceivedEventRaised = true;
      TelemetryHelper.logActionEventToAllTelemetry(LogLevel.INFO, {
        Event: TelemetryEvent.RehydrateMessageReceived,
        Description: "History message received",
        CustomProperties: payload
      });
    }
  };
  const isValidMessage = activity => {
    var _activity$channelData, _activity$channelData2, _activity$channelData3;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const messageHasNoText = !(activity !== null && activity !== void 0 && activity.text);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const messageHasNoTags = !(activity !== null && activity !== void 0 && activity.channelData) || !(activity !== null && activity !== void 0 && (_activity$channelData = activity.channelData) !== null && _activity$channelData !== void 0 && _activity$channelData.tags) || (activity === null || activity === void 0 ? void 0 : (_activity$channelData2 = activity.channelData) === null || _activity$channelData2 === void 0 ? void 0 : (_activity$channelData3 = _activity$channelData2.tags) === null || _activity$channelData3 === void 0 ? void 0 : _activity$channelData3.length) === 0;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const messageHasNoAttachments = !(activity !== null && activity !== void 0 && activity.attachments) || (activity === null || activity === void 0 ? void 0 : activity.attachments.length) === 0;
    if (messageHasNoTags && messageHasNoText && messageHasNoAttachments) {
      return false;
    }
    return true;
  };
  const receivedMessageStrategy = activity => {
    var _TelemetryManager$Int4;
    if (!isValidMessage(activity)) return;
    const isHistoryMessageReceived = isHistoryMessage(activity, startTime);
    const payload = buildMessagePayload(activity, userId);
    payload.messageType = Constants.userMessageTag;
    if (isHistoryMessageReceived) {
      var _TelemetryManager$Int3;
      historyMessageStrategy(polyfillMessagePayloadForEvent(activity, payload, (_TelemetryManager$Int3 = TelemetryManager.InternalTelemetryData) === null || _TelemetryManager$Int3 === void 0 ? void 0 : _TelemetryManager$Int3.conversationId));
      return;
    }
    firstResponseLatencyTracker.stopClock(payload);
    const newMessageReceivedEvent = {
      eventName: BroadcastEvent.NewMessageReceived,
      payload: polyfillMessagePayloadForEvent(activity, payload, (_TelemetryManager$Int4 = TelemetryManager.InternalTelemetryData) === null || _TelemetryManager$Int4 === void 0 ? void 0 : _TelemetryManager$Int4.conversationId)
    };
    BroadcastService.postMessage(newMessageReceivedEvent);
    TelemetryHelper.logActionEventToAllTelemetry(LogLevel.INFO, {
      Event: TelemetryEvent.MessageReceived,
      Description: "New message received",
      CustomProperties: payload
    });
  };
  const raiseMessageEvent = activity => {
    if ((activity === null || activity === void 0 ? void 0 : activity.type) === Constants.message) {
      const scenarioType = getScenarioType(activity);
      switch (scenarioType) {
        case ScenarioType.UserSendMessageStrategy:
          userSendMessageStrategy(activity);
          break;
        case ScenarioType.SystemMessageStrategy:
          systemMessageStrategy(activity);
          break;
        case ScenarioType.ReceivedMessageStrategy:
          receivedMessageStrategy(activity);
          break;
      }
    }
  };
  return onNewAdapterActivityHandler;
};